
import inspect

from aiohttp import web

from colibris import api
from colibris import authentication
from colibris import authorization
from colibris import views
from colibris.authorization.permissions import get_required_permissions


@web.middleware
async def handle_auth(request, handler):
    # Bail out with exception if we haven't gotten proper match_info
    if request.match_info.http_exception is not None:
        raise request.match_info.http_exception

    request = authentication.process_request(request)

    method = request.method
    path = request.match_info.route.resource.canonical

    original_handler = request.match_info.handler

    # First look for required permissions in the view handler itself (class or function)
    required_permissions = get_required_permissions(original_handler)

    # Then, if we've got a class-based view, look for required permissions in method function
    if inspect.isclass(original_handler) and issubclass(original_handler, views.View):
        method_func = getattr(original_handler, method.lower(), None)
        if method_func:
            method_func_required_permissions = get_required_permissions(method_func)
            if method_func_required_permissions and required_permissions:
                required_permissions = required_permissions.combine(method_func_required_permissions)

            else:
                required_permissions = required_permissions or method_func_required_permissions

    try:
        account = authentication.authenticate(request)

    except authentication.AuthenticationException:
        if required_permissions is not None:
            raise api.UnauthenticatedException()

        account = None

    if required_permissions is not None:
        try:
            authorization.authorize(account, method, path, original_handler, required_permissions)

        except authorization.AuthorizationException:
            raise api.ForbiddenException()

    response = await handler(request)
    response = authentication.process_response(request, response)

    return response



import pytest

from aiohttp import web, hdrs
from marshmallow import Schema, fields

from colibris.middleware.errors import handle_errors_json
from colibris.views import APIView


class ItemSchema(Schema):
    name = fields.String()
    info = fields.String()
    count = fields.Integer()


class QuerySchema(Schema):
    count = fields.Integer()


class ItemsView(APIView):
    body_schema_class = ItemSchema
    query_schema_class = QuerySchema

    async def get(self):
        query = await self.get_validated_query()

        return web.json_response({'query': query})

    async def post(self):
        query = await self.get_validated_query()
        body = await self.get_validated_body()

        return web.json_response({'query': query, 'body': body})


@pytest.fixture
def http_client(loop, aiohttp_client):
    middlewares = [
        handle_errors_json
    ]

    app = web.Application(middlewares=middlewares)
    app.router.add_route('*', '/items', ItemsView)

    return loop.run_until_complete(aiohttp_client(app))


async def test_get(http_client):
    sent_args = {
        'count': '222'
    }
    expected_args = {
        'count': 222
    }

    response = await http_client.request(hdrs.METH_GET, '/items', params=sent_args)

    assert response.status == 200

    data = await response.json()
    query = data['query']

    assert expected_args == query


async def test_post_body(http_client):
    sent_data = {
        'name': 'Risus Fusce',
        'info': 'Egestas Lorem Sit Fringilla',
        'count': '22',
    }
    expected_data = {
        'name': 'Risus Fusce',
        'info': 'Egestas Lorem Sit Fringilla',
        'count': 22,
    }

    response = await http_client.request(hdrs.METH_POST, '/items', json=sent_data)

    assert response.status == 200

    data = await response.json()
    body = data['body']

    assert expected_data == body


async def test_post_query(http_client):
    sent_data = {
        'count': '22',
    }
    expected_data = {
        'count': 22,
    }

    response = await http_client.request(hdrs.METH_POST, '/items', params=sent_data)

    assert response.status == 200

    data = await response.json()
    query = data['query']

    assert expected_data == query



from colibris import persist


# Here are some examples of models. Just remove what you don't need.

class User(persist.Model):
    id = persist.AutoField()
    username = persist.CharField(max_length=128, index=True, unique=True)
    password = persist.CharField(max_length=128)
    first_name = persist.CharField(max_length=64)
    last_name = persist.CharField(max_length=64)
    email = persist.CharField(max_length=128, null=True)


class Right(persist.Model):
    id = persist.AutoField()
    user = persist.ForeignKeyField(User)
    resource = persist.CharField(max_length=128)
    operations = persist.CharField(max_length=16)
